package pl.edu.icm.synat.process.common.enrich.impl.sonca;

/*
 * #%L
 * Synat: 
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import static pl.edu.icm.synat.logic.model.utils.YModelUtils.AT_CLASSIFICATION_OECD_BY;

import org.apache.commons.lang.ArrayUtils;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YTagList;
import pl.edu.icm.model.bwmeta.y.constants.TagTypes;

/**
 * {@code Keywords classification} based meta data transformer.
 * Use enriched {@code YElement} to update meta data keywords in original {@code YElement}.
 * Provides {@code canBeModify} and {@code modifyIfNeeded} methods.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class KeywordsSoncaMetadataTransformer {
    private final static int BEG = 0;
    private final static String TG_SONCA_KEYWORDS = "sonca.keywords";

    private int maxKeywords = 6;

    public KeywordsSoncaMetadataTransformer(int maxKeywords) {
        this.maxKeywords = maxKeywords;
    }

    public KeywordsSoncaMetadataTransformer() {
    }

    /**
     * Checks is {@code YElement} changeable.
     * @param element, input {@code YElement}.
     * @return {@code true} if meta data can be modified
     *  {@code false} otherwise.
     */
    public boolean canBeModify(YElement element) {
        return ( ! hasKeywords(element));
    }

    /**
     * Modifies input {@code element} {@code YElement} if {@code enrichedYElement} {@code YElement}
     * is enriched. The {@code YTagList} with {@code TagTypes.TG_KEYWORD}.
     * The {@code maxKeywords} is used to estimate {@code YTagList} {@code YTagList#getValues()} amount.
     * @param element, will be updated with classification.
     * @param enrichedYElement, should has classification.
     * @return {@code true} if meta data was modify
     *  {@code false} otherwise.
     */
    protected boolean modifyIfNeeded(YElement element, final YElement enrichedYElement) {
        if ( ! enrichedYElement.getTagLists(TG_SONCA_KEYWORDS).isEmpty()) {
            prepareKeywords(enrichedYElement, TG_SONCA_KEYWORDS, element, TagTypes.TG_KEYWORD);
            element.addAttribute(AT_CLASSIFICATION_OECD_BY, "Sonca");
            return true;
        }
        return false;
    }

    private boolean hasKeywords(YElement element) {
        if (element.getTagLists().isEmpty()) {
            return false;
        }
        if (element.getTagLists(TagTypes.TG_KEYWORD).isEmpty()) {
            return false;
        }
        return true;
    }

    private void prepareKeywords(YElement enrichedElement, final String from, YElement element, final String to) {
        if (element.getTagLists(to).isEmpty()) {
            for (YTagList tags : enrichedElement.getTagLists(from)) {
                prepareElementKeywords(element, to, tags);
            }
        } else {
            prepareTagsWithExistedTags(enrichedElement, from, element, to);
        }
    }

    private void prepareTagsWithExistedTags(YElement enrichedElement,
            final String from, YElement element, final String to) {
        for (YTagList tags : enrichedElement.getTagLists(from)) {
            for (YTagList orgTags : element.getTagLists(to)) {
                if ( ! tags.getLanguage().equals(orgTags.getLanguage())) {
                    continue;
                }
                prepareElementKeywords(element, to, tags, orgTags);
            }
        }
    }

    private void prepareElementKeywords(YElement element, final String to, YTagList tags) {
        tags.setType(to);
        resizeAndSetValues(tags, maxKeywords);
        element.addTagList(tags);
    }

    private void prepareElementKeywords(YElement element, final String to, YTagList tags, YTagList orgTags) {
        Object[] orgValuesArray = orgTags.getValues().toArray();
        for (String tagValue : tags.getValues()) {
            if (ArrayUtils.contains(orgValuesArray, tagValue)) {
                continue;
            }
            orgTags.addValue(tagValue);
        }
        resizeAndSetValues(orgTags, maxKeywords);
    }

    private void resizeAndSetValues(YTagList orgTags, final int maxSize) {
    	if (orgTags.getValues().size() < maxSize) {
    		orgTags.setValues(orgTags.getValues());
    	} else {
	        orgTags.setValues(orgTags.getValues().subList(BEG, maxSize));
    	}
    }

    public void setMaxKeywords(int maxKeywords) {
        this.maxKeywords = maxKeywords;
    }
}
