package pl.edu.icm.synat.process.common.enrich.impl.sonca;

/*
 * #%L
 * Synat: 
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import static pl.edu.icm.synat.logic.model.utils.YModelUtils.AT_CLASSIFICATION_OECD_BY;
import static pl.edu.icm.synat.logic.model.utils.YModelUtils.OECD_CLASSIFICATION;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.ArrayUtils;

import pl.edu.icm.model.bwmeta.y.YCategoryRef;
import pl.edu.icm.model.bwmeta.y.YElement;

/**
 * {@code Sonca/OECD classification} based meta data transformer.
 * Use enriched {@code YElement} to update meta data OECD classification in original {@code YElement}.
 * Provides {@code canBeModify} and {@code modifyIfNeeded} methods.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class OecdSoncaMetadataTransformer {
    private final static String SONCA_OECD_CLASSIFICATION_ID = "bwmeta1.category-class.sonca.OECD";

    public OecdSoncaMetadataTransformer() {
    }

    /**
     * Checks is {@code YElement} changeable.
     * @param element, input {@code YElement}.
     * @return {@code true} if meta data can be modified
     *  {@code false} otherwise.
     */
    public boolean canBeModify(YElement element) {
        return ( ! hasClassification(element));
    }

    /**
     * Modifies input {@code element} {@code YElement} if {@code enrichedYElement} {@code YElement}
     * is enriched. The {@code YAttribute} is setup with {@code key} {@code AT_CLASSIFICATION_OECD_BY}
     *  and  {@code value} {@code "Sonca"}.
     * @param element, will be updated with classification.
     * @param enrichedYElement, should has classification.
     * @return {@code true} if meta data is modified
     *  {@code false} otherwise.
     */
    protected boolean modifyIfNeeded(YElement element, final YElement enrichedYElement) {
        if (hasClassification(enrichedYElement, SONCA_OECD_CLASSIFICATION_ID)) {
            changeCategoryRef(enrichedYElement, SONCA_OECD_CLASSIFICATION_ID, element, OECD_CLASSIFICATION);
            element.addAttribute(AT_CLASSIFICATION_OECD_BY, "Sonca");
            return true;
        }
        return false;
    }

    private boolean hasClassification(YElement element) {
        if ( ! element.getAttributes(AT_CLASSIFICATION_OECD_BY).isEmpty()) {
            return true;
        }
        return hasClassification(element, OECD_CLASSIFICATION);
    }

    private void changeCategoryRef(YElement elementWithCode, final String from, YElement element, final String to) {
        final Map<String, String> rootCategories = new HashMap<String, String>();
        changeCategoryRef(element, elementWithCode, from, to, rootCategories);
        for (String code : rootCategories.keySet()) {
            element.addCategoryRef(new YCategoryRef(to, code));
        }
    }

    private boolean hasClassification(final YElement element, final String... classifcations) {
        for (final YCategoryRef yCategoryRef : element.getCategoryRefs()) {
            if (ArrayUtils.contains(classifcations, yCategoryRef.getClassification())) {
                return true;
            }
        }
        return false;
    }

    private void changeCategoryRef(YElement element, YElement elementWithCode, final String from, final String to
            , final Map<String, String> rootCategories) {
        for (final YCategoryRef yCategoryRefWithCode : elementWithCode.getCategoryRefs()) {
            if (null != yCategoryRefWithCode && from.equals(yCategoryRefWithCode.getClassification())) {
                chengeCategoryRef(element, to, rootCategories, yCategoryRefWithCode);
            }
        }
    }

    private void chengeCategoryRef(YElement element, final String to, final Map<String, String> rootCategories
            , final YCategoryRef yCategoryRefWithCode) {
        YCategoryRef yCategoryRefToModify = new YCategoryRef();
        yCategoryRefToModify.setClassification(to);
        yCategoryRefToModify.setCode(yCategoryRefWithCode.getCode());
        element.addCategoryRef(yCategoryRefToModify);
        addRootCategories(yCategoryRefToModify, rootCategories);
    }

    private void addRootCategories(YCategoryRef yCategoryRef, Map<String, String> rootCategories) {
        String[] categoriesParts = yCategoryRef.getCode().split("\\.");
        rootCategories.put(categoriesParts[0], "");
    }
}
