package pl.edu.icm.synat.process.common.enrich.impl.sonca;

/*
 * #%L
 * Synat: 
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.util.Collections;
import java.util.List;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.synat.process.common.enrich.MetadataTransformer;
import pl.edu.icm.synat.process.common.enrich.type.SoncaEnrichType;


/**
 * {@code Sonca/OECD} based classification and/or keywords meta data transformer.
 * Implementation of {@code MetadataTransformer}.
 * Use {@code SoncaMetadataTransformer} and/or {@code KeywordsMetadataTransformer}.
*
* forceEnrich, {@code boolean} with default value true, used to force enriching;
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class SoncaAllMetadataTransformer implements MetadataTransformer {
	/**
	 * {@code boolean} with default value true, used to force enriching;
	 */
    private boolean forceEnrich = false;
    private OecdSoncaMetadataTransformer oecdSoncaMetadataTransformer;
    private KeywordsSoncaMetadataTransformer keywordsSoncaMetadataTransformer;
    private SoncaMetadataEnricher metadataEnricher;

    public SoncaAllMetadataTransformer(OecdSoncaMetadataTransformer oecdMetadataTransformer
            , KeywordsSoncaMetadataTransformer keywordMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher, boolean forceEnrich) {
        this.oecdSoncaMetadataTransformer = oecdMetadataTransformer;
        this.keywordsSoncaMetadataTransformer = keywordMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
        this.forceEnrich = forceEnrich;
    }

    /**
     * Remember to use only with {@code SoncaEnrichType.OECD_ONLY} null pointer on
     *  {@code KeywordsSoncaMetadataTransformer} is not checked.
     * @param oecdMetadataTransformer
     * @param metadataEnricher
     * @param forceEnrich
     */
    public SoncaAllMetadataTransformer(OecdSoncaMetadataTransformer oecdMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher, boolean forceEnrich) {
        this.oecdSoncaMetadataTransformer = oecdMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
        this.forceEnrich = forceEnrich;
    }

    /**
     * Remember to use only with {@code SoncaEnrichType.OECD_ONLY} null pointer on
     *  {@code KeywordsSoncaMetadataTransformer} is not checked.
     * @param oecdMetadataTransformer
     * @param metadataEnricher
     */
    public SoncaAllMetadataTransformer(OecdSoncaMetadataTransformer oecdMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher) {
        this.oecdSoncaMetadataTransformer = oecdMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
    }

    /**
     * Remember to use only with {@code SoncaEnrichType.KEYWORDS_ONLY} null pointer on
     *  {@code SoncaMetadataTransformer} is not checked.
     * @param keywordMetadataTransformer
     * @param metadataEnricher
     * @param forceEnrich
     */
    public SoncaAllMetadataTransformer(KeywordsSoncaMetadataTransformer keywordMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher, boolean forceEnrich) {
        this.keywordsSoncaMetadataTransformer = keywordMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
        this.forceEnrich = forceEnrich;
    }

    /**
     * Remember to use only with {@code SoncaEnrichType.KEYWORDS_ONLY} null pointer on
     *  {@code SoncaMetadataTransformer} is not checked.
     * @param keywordMetadataTransformer
     * @param metadataEnricher
     */
    public SoncaAllMetadataTransformer(KeywordsSoncaMetadataTransformer keywordMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher) {
        this.keywordsSoncaMetadataTransformer = keywordMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
    }

    public SoncaAllMetadataTransformer(OecdSoncaMetadataTransformer oecdMetadataTransformer
            , KeywordsSoncaMetadataTransformer keywordMetadataTransformer
            , SoncaMetadataEnricher metadataEnricher) {
        this.oecdSoncaMetadataTransformer = oecdMetadataTransformer;
        this.keywordsSoncaMetadataTransformer = keywordMetadataTransformer;
        this.metadataEnricher = metadataEnricher;
    }

    public SoncaAllMetadataTransformer(OecdSoncaMetadataTransformer oecdMetadataTransformer
            , KeywordsSoncaMetadataTransformer keywordMetadataTransformer) {
        this.oecdSoncaMetadataTransformer = oecdMetadataTransformer;
        this.keywordsSoncaMetadataTransformer = keywordMetadataTransformer;
    }

    public SoncaAllMetadataTransformer() {
    }

    @Override
    public boolean modify(YElement element) {
        if (SoncaEnrichType.OECD_ONLY.equals(metadataEnricher.getEnrichingtype())) {
            return prepareOecdMetadata(element);
        }
        if (SoncaEnrichType.KEYWORDS_ONLY.equals(metadataEnricher.getEnrichingtype())) {
            return prepareKeywords(element);
        }
        return prepareOecdAndKeywords(element);
    }

    /**
     * Method prepares the {@code YElement} for enriching.
     * At {@code SoncaAllMetadataTransformer} level is just single list entry.
     * But on {@code SoncaMetadataWithChildrenTransformer} it's {@code List} of {@code YElement}s
     *  for all inherited meta data elements. 
     * @param element, input {@code YElement} witch will be enriched.
     * @return, {@code List} of {@code YElement}s as a {@code MetadataEnricher} input data
     *  with a input {@code YElement}.
     */
    protected List<YElement> prepareYElement(YElement element) {
        return Collections.singletonList(element);
    }

    private boolean prepareOecdAndKeywords(YElement element) {
        boolean shouldEnrichKeywords = (forceEnrich
                || keywordsSoncaMetadataTransformer.canBeModify(element));
        boolean shouldEnrichOecd = (forceEnrich
                || oecdSoncaMetadataTransformer.canBeModify(element));
        YElement enrichedYElement = null;
        if (shouldEnrichKeywords || shouldEnrichOecd) {
            enrichedYElement = metadataEnricher.enrich(prepareYElement(element));
        }
        boolean didEnrichKeywords = false;
        if (shouldEnrichKeywords) {
            didEnrichKeywords = keywordsSoncaMetadataTransformer.modifyIfNeeded(element
                    , enrichedYElement);
        }
        boolean didEnrichOecd = false;
        if (shouldEnrichOecd) {
            didEnrichOecd = oecdSoncaMetadataTransformer.modifyIfNeeded(element, enrichedYElement);
        }

        return didEnrichKeywords || didEnrichOecd;
    }

    private boolean prepareKeywords(YElement element) {
        return (forceEnrich || keywordsSoncaMetadataTransformer.canBeModify(element))
                && keywordsSoncaMetadataTransformer.modifyIfNeeded(element
                        , metadataEnricher.enrich(prepareYElement(element)));
    }

    private boolean prepareOecdMetadata(YElement element) {
        return (forceEnrich || oecdSoncaMetadataTransformer.canBeModify(element))
                && oecdSoncaMetadataTransformer.modifyIfNeeded(element
                        , metadataEnricher.enrich(prepareYElement(element)));
    }

    public void setForceEnrich(Boolean forceEnrich) {
        this.forceEnrich = null != forceEnrich && forceEnrich;
    }

    public void setOecdSoncaMetadataTransformer(OecdSoncaMetadataTransformer oecdSoncaMetadataTransformer) {
        this.oecdSoncaMetadataTransformer = oecdSoncaMetadataTransformer;
    }

    public void setKeywordsSoncaMetadataTransformer(KeywordsSoncaMetadataTransformer keywordsSoncaMetadataTransformer) {
        this.keywordsSoncaMetadataTransformer = keywordsSoncaMetadataTransformer;
    }

    public void setMetadataEnricher(SoncaMetadataEnricher metadataEnricher) {
        this.metadataEnricher = metadataEnricher;
    }
}
