package pl.edu.icm.synat.process.common.enrich.impl.sonca;

/*
 * #%L
 * Synat: 
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import static pl.edu.icm.synat.logic.index.IndexFieldConstants.SEP;
import static pl.edu.icm.synat.logic.index.publication.PublicationIndexFieldConstants.FIELD_ANCESTOR_ID;
import static pl.edu.icm.synat.logic.index.publication.PublicationIndexFieldConstants.FIELD_CONFERENCE_EVENT_ID;
import static pl.edu.icm.synat.logic.index.publication.PublicationIndexFieldConstants.FIELD_DESCRIPTION;
import static pl.edu.icm.synat.logic.index.publication.PublicationIndexFieldConstants.FIELD_EXID;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YStructure;
import pl.edu.icm.model.bwmeta.y.constants.HierarchyWithLevelIds;
import pl.edu.icm.synat.api.services.index.fulltext.FulltextIndexService;
import pl.edu.icm.synat.api.services.index.fulltext.query.FulltextSearchQuery;
import pl.edu.icm.synat.api.services.index.fulltext.query.Order;
import pl.edu.icm.synat.api.services.index.fulltext.query.criteria.SearchCriterion;
import pl.edu.icm.synat.api.services.index.fulltext.query.criteria.SearchOperator;
import pl.edu.icm.synat.api.services.index.fulltext.query.criteria.impl.FieldCriterion;
import pl.edu.icm.synat.api.services.index.fulltext.query.criteria.impl.FieldRangeCriterion;
import pl.edu.icm.synat.api.services.index.fulltext.query.format.FieldRequest;
import pl.edu.icm.synat.api.services.index.fulltext.query.format.ResultsFormat;
import pl.edu.icm.synat.api.services.index.fulltext.result.FulltextSearchResult;
import pl.edu.icm.synat.api.services.index.fulltext.result.ResultField;
import pl.edu.icm.synat.api.services.store.StatelessStore;
import pl.edu.icm.synat.api.services.store.model.Record;
import pl.edu.icm.synat.api.services.store.model.RecordId;
import pl.edu.icm.synat.logic.model.utils.RecordBwmetaExtractor;
import pl.edu.icm.synat.process.common.enrich.MetadataTransformer;

/**
 * Includes all children elements of {@code YElement} on classification.
 * {@code Sonca/OECD classification} based meta data transformer.
 * Implementation of {@code MetadataTransformer} extends {@code SoncaMetadataTransformer}
 * for enriching detection and classification informations updating.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class SoncaMetadataWithChildrenTransformer extends SoncaAllMetadataTransformer implements MetadataTransformer {
    private final static String ALL = "*";

	/**
     * Array used to prepare articles/journals related articles.
     */
    private final static String[] ELEMENT_LEVELS = new String[] {
        HierarchyWithLevelIds.EXT_LEVEL_BOOK_SERIES
        , HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_JOURNAL
        , HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_SERIES
        , HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_YEAR
        , HierarchyWithLevelIds.EXT_LEVEL_JOURNAL_VOLUME
    };

    /**
     * Array used to prepare conference related articles.
     */
    private final static String[] CONFERENCE_ELEMENT_LEVELS = new String[] {
        HierarchyWithLevelIds.EXT_LEVEL_CONFERENCE_EVENT
    };

    /**
     * Search query size limitation. 
     */
    private int searchedElementSize = 10;
    private FulltextIndexService indexService = null;
    private StatelessStore store = null;
    private RecordBwmetaExtractor recordBwmetaExtractor = null;

    /**
     * The {@code SoncaMetadataWithChildrenTransformer} needs collect meta data from all
     *  {@code YElemet} children.  prepareChildren uses {@code FulltextIndexService}
     *  and {@code StatelessStore} to collect those informations.
     * @param element, {@code YElemet} should have information about this children.
     * @return, {@code List} of {@code YElement}s as a {@code MetadataEnricher} input data
     *  with correct children structure and input {@code YElement}.
     */
    protected List<YElement> prepareYElement(YElement element) {
        final List<YElement> elements = new ArrayList<YElement>();
        elements.add(element);
        for (final YStructure structure : element.getStructures()) {
            if (structure.getCurrent() != null && ArrayUtils.contains(CONFERENCE_ELEMENT_LEVELS
                    , structure.getCurrent().getLevel())) {
                elements.addAll(findConferenceChildrenByLevel(element, structure.getCurrent().getLevel()));
            } else if (structure.getCurrent() != null && ArrayUtils.contains(ELEMENT_LEVELS, structure.getCurrent().getLevel())) {
                elements.addAll(findChildrenByLevel(element, structure.getCurrent().getLevel()));
            }
        }
        return elements;
    }

    private List<YElement> findChildrenByLevel(YElement element, String level) {
        return prepareChildrenByLevel(prepareSearchQuery(prepareFieldCriterionList(element, level)));
    }

    private List<YElement> findConferenceChildrenByLevel(YElement element, String level) {
        return prepareChildrenByLevel(prepareSearchQuery(prepareConferenceFieldCriterionList(element)));
    }

    private List<YElement> prepareChildrenByLevel(final FulltextSearchQuery searchQuery) {
        List<YElement> elements = new ArrayList<YElement>();
        for (final FulltextSearchResult fulltextSearchResult : indexService.performSearch(searchQuery).getResults()) {
            for (final ResultField resultField : fulltextSearchResult.getFields()) {
                if (FIELD_EXID.equals(resultField.getName())) {
                    YElement yElement = fetchYElement(resultField.getValues()[0]);
                    if (null != yElement) {
                        elements.add(yElement);
                    }
                }
            }
        }
        return elements;
    }

    private List<SearchCriterion> prepareFieldCriterionList(YElement element, String level) {
        final List<SearchCriterion> fieldCriterionList = new ArrayList<SearchCriterion>();
        fieldCriterionList.add(new FieldCriterion(new StringBuilder(FIELD_ANCESTOR_ID)
                .append(SEP).append(level).toString(), element.getId(), SearchOperator.AND));
        return fieldCriterionList;
    }

    private List<SearchCriterion> prepareConferenceFieldCriterionList(YElement element) {
        final List<SearchCriterion> fieldCriterionList = new ArrayList<SearchCriterion>();
        fieldCriterionList.add(new FieldCriterion(new StringBuilder(FIELD_CONFERENCE_EVENT_ID)
                .toString(), element.getId(), SearchOperator.AND));
        return fieldCriterionList;
    }

    private FulltextSearchQuery prepareSearchQuery(final List<SearchCriterion> fieldCriterionList) {
        final SearchCriterion fieldRangeCriterion = new FieldRangeCriterion(FIELD_DESCRIPTION, ALL, ALL);
        fieldRangeCriterion.setOperator(SearchOperator.AND);
        fieldCriterionList.add(fieldRangeCriterion);
        
        final List<Order> orderList = new ArrayList<Order>();
        orderList.add(Order.relevanceOrder());

        final ResultsFormat resultsFormat = new ResultsFormat(new FieldRequest(FIELD_EXID, true));
        return new FulltextSearchQuery(0, searchedElementSize, resultsFormat, orderList, fieldCriterionList);
    }

    private YElement fetchYElement(final String id) { 
        final Record record = store.fetchRecord(new RecordId(id));
        if (null != record && ! record.isDeleted()) {
            return recordBwmetaExtractor.extractElement(record);
        }
        return null;
    }

    public void setSearchedElementSize(int searchedElementSize) {
        this.searchedElementSize = searchedElementSize;
    }

    public void setIndexService(FulltextIndexService indexService) {
        this.indexService = indexService;
    }

    public void setStore(StatelessStore store) {
        this.store = store;
    }

    public void setRecordBwmetaExtractor(RecordBwmetaExtractor recordBwmetaExtractor) {
        this.recordBwmetaExtractor = recordBwmetaExtractor;
    }
}
