package pl.edu.icm.synat.process.common.harvesting.impl;

/*
 * #%L
 * Synat: harvester License.
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.regex.PatternSyntaxException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import pl.edu.icm.model.bwmeta.y.YCategoryRef;
import pl.edu.icm.synat.logic.model.utils.YModelUtils;
import pl.edu.icm.synat.process.common.harvesting.SubjectDisciplineMapper;

/**
 * Subject discipline mapper implementation of {@code SubjectDisciplineMapper}.
 * Implementation is base on {@code Properties} where key is subject and value is list of disciplines
 * separated by comma.
*
* @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
*
*/
public class SimpleSubjectDisciplineMapper implements SubjectDisciplineMapper {
    private Properties properties;

    /**
     * Expected is that {@code Properties} key are whitespace less and low cased all keys.
     * Algorithm work is unexpected if clean subjects are read to {@code Properties}
     * without correct/expected form.
     * 
     * @param properties, {@code Properties} object with cleaned all whitespaces and low cased all keys.
     */
    public SimpleSubjectDisciplineMapper(Properties properties) {
        this.setProperties(properties);
    }

    public SimpleSubjectDisciplineMapper(String filepath) throws PatternSyntaxException, IllegalArgumentException, IOException, URISyntaxException{
        this(prepareCorrectPropertiesFromFile(new File(ClassLoader.getSystemResource(filepath).toURI())));
    }
    
    @Override
    public List<YCategoryRef> convertSubjects(List<String> subjects) {
        List<YCategoryRef> yCategoryRefs = new ArrayList<YCategoryRef>();
        for (String subject : subjects) {
            String tested = subject.replaceAll(" ", "").toLowerCase();        
            if (!properties.containsKey(tested)) {
                // TODO add logging, when subjects are out of Properties
                continue;
            }
            addDisciplines(yCategoryRefs, properties.getProperty(tested));
        }
        prepareRootDisciplines(yCategoryRefs);
        return yCategoryRefs;
    }

    /**
     * Prepares correct {@code Properties} object without any whitespace and with all keys low cased.
     * @param file, {@code File} with IEEE subjects disciplines properties.
     * @return {@code Properties} object without any whitespace and with all keys low cased.
     * @throws IOException - in case of an I/O error, when file is read.
     * @throws IOException - if an error occurred when reading from the input stream.
     * @throws PatternSyntaxException - if the regular expression's syntax is invalid
     * @throws IllegalArgumentException - if the input stream contains a malformed Unicode escape sequence.
     */
    public static Properties prepareCorrectPropertiesFromFile(File file)
            throws IOException, PatternSyntaxException, IllegalArgumentException {
        Properties properties = new Properties();
        String all = FileUtils.readFileToString(file);
        properties.load(IOUtils.toInputStream(all.replaceAll(" ", "").toLowerCase()));
        return properties;
    }

    private void addDisciplines(List<YCategoryRef> yCategoryRefs, String property) {
        for (String discipline : property.split(",")) {
            YCategoryRef newDiscipline = new YCategoryRef(YModelUtils.OECD_CLASSIFICATION, discipline);
            if (yCategoryRefs.contains(newDiscipline)) {
                continue;
            }
            yCategoryRefs.add(newDiscipline);
        }
    }

    private void prepareRootDisciplines(List<YCategoryRef> yCategoryRefs) {
        final Map<String, String> rootCategories = new HashMap<String, String>();
        prepareRootCategories(yCategoryRefs, rootCategories);
        for (String code : rootCategories.keySet()) {
            yCategoryRefs.add(new YCategoryRef(YModelUtils.OECD_CLASSIFICATION, code));
        }
    }

    private void prepareRootCategories(List<YCategoryRef> yCategoryRefs, Map<String, String> rootCategories) {
        for (YCategoryRef yCategoryRef : yCategoryRefs) {
            String[] categoriesParts = yCategoryRef.getCode().split("\\.");
            rootCategories.put(categoriesParts[0], "");
        }
    }

    public void setProperties(Properties properties) {
        this.properties = properties;
    }

}
