package pl.edu.icm.synat.process.common.harvesting.metadata.node;

/*
 * #%L
 * Synat: .
 * %%
 * Copyright (C) 2010 - 2013 ICM, Warsaw University
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.activation.MimetypesFileTypeMap;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.edu.icm.model.bwmeta.y.YAttribute;
import pl.edu.icm.model.bwmeta.y.YCategoryRef;
import pl.edu.icm.model.bwmeta.y.YContentFile;
import pl.edu.icm.model.bwmeta.y.YDescription;
import pl.edu.icm.model.bwmeta.y.YElement;
import pl.edu.icm.model.bwmeta.y.YId;
import pl.edu.icm.model.bwmeta.y.YLanguage;
import pl.edu.icm.model.bwmeta.y.YRelation;
import pl.edu.icm.model.bwmeta.y.constants.DescriptionTypes;
import pl.edu.icm.model.bwmeta.y.constants.FileTypes;
import pl.edu.icm.model.bwmeta.y.constants.IdSchemaIds;
import pl.edu.icm.model.bwmeta.y.constants.RelationTypes;
import pl.edu.icm.model.bwmeta.y.constants.attributes.ReferenceAttributeTypes;
import pl.edu.icm.synat.logic.document.model.api.Document;
import pl.edu.icm.synat.logic.document.model.api.attachment.Attachment;
import pl.edu.icm.synat.logic.model.utils.YModelUtils;
import pl.edu.icm.synat.process.common.harvesting.HarvestingResult;
import pl.edu.icm.synat.process.common.harvesting.MetadataHarvester;
import pl.edu.icm.synat.process.common.harvesting.TitleHistory;
import pl.edu.icm.synat.process.common.harvesting.TitleHistoryElement;
import pl.edu.icm.synat.process.common.harvesting.metadata.utils.preapre.yelement.PrepareYElementUtils;
import pl.edu.icm.synat.process.common.node.SkippableNode;

/**
 * Metadata harvesting node.
 * Process {@code YElement} from {@code Document} for covers and/or metadata. 
 * Implements {@code ItemProcessor}.
 *
 * @author Gra <Gołębiewski Radosław A.> google.com/+RadoslawGolebiewski
 * @author Michal Kalisz
 *
 */
public class MetadataHarvesterNode extends SkippableNode<List<Document>> {
    private static final Logger LOGGER = LoggerFactory.getLogger(MetadataHarvesterNode.class);
    private final List<Property> properties;
    private List<MetadataHarvester> metadataHarvestersList = null;
    private PrepareYElementUtils yelementUtil = null;

    private static final String RT_PREVIOUS_TITLES = RelationTypes.RL_CONTINUATION_OF;
    private static final String RT_NEXT_TITLES = RelationTypes.RL_CONTINUATED_BY;

    public enum Property {
        HARVEST_COVER, HARVEST_ABSTRACT, HARVEST_IMPACT_FACTOR
        , HARVEST_FREQUENCY, HARVEST_CATEGORY_REFERECES, HARVEST_TITLE_HISTORY 
    }

    public MetadataHarvesterNode(List<MetadataHarvester> metadataHarvestersList) {
        this.metadataHarvestersList = new ArrayList<MetadataHarvester>();
        this.metadataHarvestersList.addAll(metadataHarvestersList);
        yelementUtil = new PrepareYElementUtils();
        properties = new ArrayList<Property>();
    }

    @Override
    public List<Document> innerProcess(final List<Document> input) {
        for (MetadataHarvester metadataHarvester : metadataHarvestersList) {
            prepareHarvestingMetadata(input, metadataHarvester);
        }
        return input;
    }

    private void prepareHarvestingMetadata(final List<Document> input, MetadataHarvester metadataHarvester) {
        for (Document document : input) {
            if ( ! metadataHarvester.isApplicable(document)) {
                continue;
            }
            HarvestingResult harvestingResult = prepareHarvestingResult(metadataHarvester, document);
            if (null == harvestingResult) {
                continue;
            }
            addCoverAndOrMetadata(document, harvestingResult);
        }
    }

    private void addCoverAndOrMetadata(Document document, HarvestingResult harvestingResult) {
        YElement element = yelementUtil.prepareYElement(document);
        if (prepareDocumentCoverAndOrMetadata(document, harvestingResult, element)) {
            document.setMetadata(element);
        }
    }

    private boolean prepareDocumentCoverAndOrMetadata(Document document, HarvestingResult harvestingResult
            , YElement element) {
        boolean metadataModified = false;
        metadataModified |= addCoverIfNeeded(document, harvestingResult, element);
        metadataModified |= addMetadataIfNeeded(harvestingResult, element);
        return metadataModified;
    }

    private boolean addMetadataIfNeeded(HarvestingResult harvestingResult, YElement element) {
        
        boolean metadataModified = false;
        
        if (shouldAbstractBeAdded(harvestingResult, element)) {
            YDescription newAbstract = new YDescription(YLanguage.Undetermined, harvestingResult.getDesciption()
                    , DescriptionTypes.DS_ABSTRACT);
            element.addDescription(newAbstract);
            metadataModified = true;
        }

        if (shouldImpactFactorBeAdded(harvestingResult, element)) {
            element.addAttribute(new YAttribute(YModelUtils.IMPACT_FACTOR
                    , harvestingResult.getImpactFactor().toEngineeringString()));
            metadataModified = true;
        }

        if (shouldFrequencyBeAdded(harvestingResult, element)) {
            element.addAttribute(new YAttribute(YModelUtils.FREQUENCY, harvestingResult.getFrequency()));
            metadataModified = true;
        }

        List<YCategoryRef> refs = element.getCategoryRefs();
        if (shouldCategoryRefsBeAdded(harvestingResult, refs)) {
            refs.addAll(harvestingResult.getCategories());
            metadataModified = true;
        }
        
        if (shouldTitlesBeAdded(harvestingResult, element)) {
            metadataModified |= addTitleHistory(harvestingResult.getOtherTitles(), element);
        }
        return metadataModified;
    }

    private boolean addTitleHistory(TitleHistory titleHistory, YElement element) {
        
        if (titleHistory == null) {
            return false;
        }

        addTitleList(element, titleHistory.getPreviousTitles(), RT_PREVIOUS_TITLES);
        addTitleList(element, titleHistory.getNextTitles(), RT_NEXT_TITLES);

        return !(element.getRelations(RT_PREVIOUS_TITLES).isEmpty() && element.getRelations(RT_NEXT_TITLES).isEmpty());
    }
    
    private void addTitleList(YElement element, List<TitleHistoryElement> titles, String relationType) {
        
        for (TitleHistoryElement titleHistoryElement : titles) {
            
            YRelation relation = new YRelation();
            relation.setType(relationType);
            
            if (titleHistoryElement.getId() != null) {
                relation.setTarget(new YId(IdSchemaIds.EXT_SCHEME_YADDA, titleHistoryElement.getId()));
            } else if (titleHistoryElement.getIssn() != null) {
            	relation.setTarget(new YId(IdSchemaIds.EXT_SCHEME_ISSN, titleHistoryElement.getIssn()));
            }
            
            String dateRange = titleHistoryElement.getDateRange();
            if (StringUtils.isNotBlank(dateRange)) {
                relation.addAttribute(ReferenceAttributeTypes.AT_REFERENCE_PARSED_DATE, dateRange);
            }
            
            String title = titleHistoryElement.getTitleName();
            if (StringUtils.isNotBlank(title)) {
                relation.addAttribute(ReferenceAttributeTypes.AT_REFERENCE_PARSED_TITLE, title);
            }
            
            element.addRelation(relation);
        }
        
    }

    private boolean shouldTitlesBeAdded(HarvestingResult harvestingResult, YElement element) {
        if (!properties.contains(Property.HARVEST_TITLE_HISTORY)) {
            return false;
        }
        
        return element.getRelations(RT_PREVIOUS_TITLES).isEmpty() && element.getRelations(RT_NEXT_TITLES).isEmpty();
    }

    private boolean shouldCategoryRefsBeAdded(HarvestingResult harvestingResult, List<YCategoryRef> refs) {
        if (!properties.contains(Property.HARVEST_CATEGORY_REFERECES)) {
            return false;
        }
        if (!refs.isEmpty()) {
            return false;
        }
        if (harvestingResult.getCategories().isEmpty()) {
            return false;
        }
        return true;
    }

    private boolean shouldFrequencyBeAdded(HarvestingResult harvestingResult, YElement element) {
        if (!properties.contains(Property.HARVEST_FREQUENCY)) {
            return false;
        }
        if (StringUtils.isEmpty(harvestingResult.getFrequency())) {
            return false;
        }
        if (!element.getAttributes(YModelUtils.FREQUENCY).isEmpty()) {
            return false;
        }
        return true;
    }

    private boolean shouldImpactFactorBeAdded(HarvestingResult harvestingResult, YElement element) {
        if (!properties.contains(Property.HARVEST_IMPACT_FACTOR)) {
            return false;
        }
        if (null == harvestingResult.getImpactFactor()) {
            return false;
        }
        if (!element.getAttributes(YModelUtils.IMPACT_FACTOR).isEmpty()) {
            return false;
        }
        return true;
    }

    private boolean shouldAbstractBeAdded(HarvestingResult harvestingResult, YElement element) {
        if (!properties.contains(Property.HARVEST_ABSTRACT)) {
            return false;
        }
        if (StringUtils.isEmpty(harvestingResult.getDesciption())) {
            return false;
        }
        if (hasAbstract(element)) {
            return false;
        }
        return true;
    }

    private boolean addCoverIfNeeded(Document document, HarvestingResult harvestingResult
            , YElement element) {
        boolean metadataModified = false;
        
        if ( ! StringUtils.isEmpty(harvestingResult.getCoverUrl()) && ! yelementUtil.hasCover(element)) {
            try {
                addCover(document, element, harvestingResult.getCoverUrl());
                metadataModified = true;
            } catch (MalformedURLException e) {
                LOGGER.info("{}", e.getMessage(), e);
            } catch (IOException e) {
                LOGGER.info("{}", e.getMessage(), e);
            }
        }
        return metadataModified;
    }

    private void addCover(Document doc, YElement element, String coverUrl) throws HttpException, IOException {
        String mimeType = prepareCoverMimeType(coverUrl);
        URL url = new URL(coverUrl);
        String fileId = url.getFile();
        if (properties.contains(Property.HARVEST_COVER)) {
            prepareCover(element, fileId, mimeType, fileId);

            byte[] data = fetchFileContent(coverUrl);
            Attachment<?> atachment = doc.addBinaryAttachment(fileId, data);
            atachment.addTagByKey("type", "source");
            atachment.addTagByKey("mime", mimeType);
        } else {
            prepareCover(element, coverUrl, mimeType, fileId);
        }
    }

    private void prepareCover(YElement element, String location, String mimeType, String fileId) {
        List<String> locations = new ArrayList<String>();
        locations.add(location);
        YContentFile newCover = new YContentFile(fileId, FileTypes.FT_COVER, mimeType, locations);
        element.getContents().add(newCover);
    }

    private String prepareCoverMimeType(String coverUrl) {
        MimetypesFileTypeMap types = new MimetypesFileTypeMap();
        String mimeType = types.getContentType(coverUrl);
        if (StringUtils.contains(mimeType, "application/octet-stream")) {
            return "image/jpeg";
        }
        return mimeType;
    }

    private byte[] fetchFileContent(String imageUrl) throws HttpException, IOException {
        GetMethod method = null;
        HttpClient httpClient = new HttpClient();
        method = new GetMethod(imageUrl);
        method.getParams().setParameter(HttpMethodParams.RETRY_HANDLER,
                new DefaultHttpMethodRetryHandler(3, false));

        if (HttpStatus.SC_OK != httpClient.executeMethod(method)) {
            throw new HttpException("Method failed: " + method.getStatusLine());
        }
        return IOUtils.toByteArray(method.getResponseBodyAsStream());
    }

    private boolean hasAbstract(YElement element) {
        boolean hasAbstract = false;
        for (YDescription entry : element.getDescriptions()) {
            if (DescriptionTypes.DS_ABSTRACT.equals(entry.getType())) {
                hasAbstract = true;
            }
        }
        return hasAbstract;
    }

    private HarvestingResult prepareHarvestingResult(MetadataHarvester metadataHarvester, Document document) {
        HarvestingResult harvestingResult = null;
        if (properties.contains(Property.HARVEST_ABSTRACT)) {
            harvestingResult = metadataHarvester.findCoverAndMetadata(document);
        } else {
            harvestingResult = metadataHarvester.findCover(document);
        }
        return harvestingResult;
    }

    public void setProperties(String properties) {
        List<String> propertiesList = Arrays.asList(properties.split(","));
        for (String property : propertiesList) {
            this.properties.add(Property.valueOf(property));
        }
    }
}
