package pl.edu.icm.synat.process.common.store.impl;

import static pl.edu.icm.synat.logic.services.repository.constants.RepositoryStoreConstants.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.util.Assert;

import pl.edu.icm.synat.api.services.store.RecordIterator;
import pl.edu.icm.synat.api.services.store.StatelessStore;
import pl.edu.icm.synat.api.services.store.model.Record;
import pl.edu.icm.synat.api.services.store.model.RecordConditions;
import pl.edu.icm.synat.api.services.store.model.RecordId;
import pl.edu.icm.synat.application.exception.GeneralBusinessException;
import pl.edu.icm.synat.process.common.store.RecordIteratorBuilder;

public class RecordIteratorBuilderImpl implements InitializingBean, RecordIteratorBuilder {

	private StatelessStore store;

//	private List<String> tags = new ArrayList<>();
	private String recordId;
	private String resumptionToken;
	private boolean created = false;

	private RecordIterator condRecordIterator;

	private String definitionId;

	private String importId;

	private String converterId;

	private String hierarchyLevel;

	private String dataSet;
	
	public void setStore(StatelessStore store) {
		this.store = store;
	}

	public void setDefinitionId(String definitionId) {
		this.definitionId = definitionId;
	}

	public void setImportId(String importId) {
		this.importId = importId;
	}

	public void setConverterId(String converterId) {
		this.converterId = converterId;
	}

	public void setHierarchyLevel(String hierarchyLevel) {
		this.hierarchyLevel = hierarchyLevel;
	}

	public void setDataSet(String dataSet) {
		this.dataSet = dataSet;
	}
	
	private void addTagIfNotEmpty(List<String> tags, String tagPrefixImportId, String value) {
		if (StringUtils.isNotBlank(value)) {
			tags.add(tagPrefixImportId+value);
		}
	}
	
	public void setRecordId(String recordId) {
		if (StringUtils.isNotBlank(recordId)) {
			this.recordId = recordId;
		}
	}
	
	@Override
	public void afterPropertiesSet() throws Exception {
		Assert.notNull(store, "Store must be not null");
	}

	@Override
	public long calculateExpectedSize() {
		
		long expectedSize = 0;
		
		if (recordId != null) {
			expectedSize=countByRecordId();
		} else {
			RecordConditions conditions = createRecordConditions();
			expectedSize += store.countRecords(conditions, resumptionToken);
		}
		
		return expectedSize;
	}
	
	private long countByRecordId() {
        return recordId.split(",").length;
    }

    @Override
	public Iterator<Record> build() {
		
		if (created) {
			throw new GeneralBusinessException("Iterator has been already build!");
		}
		
		Iterator<Record> result = null;
		RecordConditions recordConditions = createRecordConditions();

		if (recordId != null) {
			result = buildRecordsByIds();
		} else if (recordConditions != null) {
			condRecordIterator = new RecordIterator(store, recordConditions, resumptionToken, 1000);
			result = condRecordIterator;
		}

		if (result == null) {
			throw new IllegalStateException("No condition for particular recordId nor for tags");
		}
		
		created = true;
		
		return result;
	}

    private Iterator<Record> buildRecordsByIds() {
        Iterator<Record> result;

        String[] ids = recordId.split(",");
        List<RecordId> recordIds = new ArrayList<RecordId>();
        for (String id : ids) {
            recordIds.add(new RecordId(id));
        }
        result = store.fetchRecords(recordIds).iterator();
        
        return result;
    }

	protected RecordConditions createRecordConditions() {
		RecordConditions recordConditions = null;
		List<String> tags = prepareTags();
		if (!tags.isEmpty()) {
			recordConditions = new RecordConditions();
			recordConditions.getTags().addAll(tags);
		} else if (recordId == null) {
			recordConditions = new RecordConditions();
		}
		return recordConditions;
	}

	private List<String> prepareTags() {
		
		List<String> tags = new ArrayList<>();
		
		if (StringUtils.isNotBlank(importId)) {
			addTagIfNotEmpty(tags, TAG_PREFIX_IMPORT, importId );
		} else {
			addTagIfNotEmpty(tags, TAG_PREFIX_IMPORT_DATA_SOURCE_DEFINITIONS, definitionId );
		}
		addTagIfNotEmpty(tags, TAG_PREFIX_CONVERTER, converterId);
		addTagIfNotEmpty(tags, TAG_PREFIX_HIERARHY_LEVEL, hierarchyLevel);
		addTagIfNotEmpty(tags, TAG_PREFIX_DATASET, dataSet);

		return tags;
	}

	@Override
	public String getLastToken() {
		if (condRecordIterator != null) {
			return condRecordIterator.getLastToken();
		}
		return null;
	}

	@Override
	public void setResumptionToken(String resumptionToken) {
		this.resumptionToken = resumptionToken;
	}
	
}
