package pl.edu.icm.synat.process.common.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.apache.solr.update.processor.LangDetectLanguageIdentifierUpdateProcessor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.cybozu.labs.langdetect.Detector;
import com.cybozu.labs.langdetect.DetectorFactory;
import com.cybozu.labs.langdetect.LangDetectException;
import com.cybozu.labs.langdetect.Language;
import com.google.common.collect.Sets;

import pl.edu.icm.model.bwmeta.y.YLanguage;
import pl.edu.icm.synat.application.exception.GeneralBusinessException;

public class LangDetectUtil {

	private static boolean loaded;
	private static final float MINIMUM_PROBABILITY = 0.95f;
	private static final float MINIMUM_PROBABILITY_FROM_SET = 0.75f;
	private static final Logger log = LoggerFactory.getLogger(LangDetectUtil.class);
	private static final Set<YLanguage> detectableLanguages = Sets.immutableEnumSet(YLanguage.Polish, YLanguage.English, YLanguage.Spanish, YLanguage.French,
			YLanguage.German, YLanguage.Italian, YLanguage.Russian, YLanguage.Czech, YLanguage.Hungarian);
	private static final Set<YLanguage> supportedLanguages = Sets.immutableEnumSet(YLanguage.Polish, YLanguage.English);
	private static final Set<YLanguage> unknownLanguages = Sets.immutableEnumSet(YLanguage.Undetermined, YLanguage.NoLinguisticContent);

	public static synchronized void loadData() {
		if (loaded) {
			return;
		}
		loaded = true;
		List<String> profileData = new ArrayList<String>();
		try {
			Charset encoding = Charset.forName("UTF-8");
			for (YLanguage language : detectableLanguages) {
				InputStream stream = LangDetectLanguageIdentifierUpdateProcessor.class.getResourceAsStream("langdetect-profiles/" + language.getShortCode());
				BufferedReader reader = new BufferedReader(new InputStreamReader(stream, encoding));

				profileData.add(new String(IOUtils.toCharArray(reader)));

				reader.close();
			}
			DetectorFactory.loadProfile(profileData);
			DetectorFactory.setSeed(System.currentTimeMillis());
		} catch (IOException | LangDetectException e) {
			throw new GeneralBusinessException(e);
		}
	}

	public static boolean isSupported(YLanguage language) {
		return supportedLanguages.contains(language);
	}

	public static boolean isUndefined(YLanguage language) {
		return unknownLanguages.contains(language);
	}

	public static YLanguage processLanguage(Collection<String> inputs, YLanguage currentLanguage) {
		loadData();
		boolean inputsEmpty = true;
		for (String input : inputs) {
			inputsEmpty = inputsEmpty && input.isEmpty();
		}
		if (currentLanguage.getShortCode().isEmpty() && unknownLanguages.contains(currentLanguage) && !inputsEmpty) {
			try {
				Detector detector = DetectorFactory.create(0.5);
				for (String input : inputs) {
					detector.append(input);
				}
				for (Language lang : detector.getProbabilities()) {
					YLanguage yLang = YLanguage.byCode(lang.lang);
					if (isSupported(yLang))
						return yLang;
				}
				return currentLanguage;
			} catch (LangDetectException e) {
				log.debug("Couldn't determine content language", e);
			}
		}
		return currentLanguage;
	}
	
	public static YLanguage getLanguage(String text, Set<YLanguage> possibleLanguages) {
		try {
			Detector detector = DetectorFactory.create(0.5f);
			detector.append(text);
			return detectLanguage(possibleLanguages, detector);
		} catch (LangDetectException e) {
			log.debug("Couldn't determine content language", e);
			return YLanguage.Undetermined;
		}
	}
	
	public static YLanguage getLanguage(Reader text, Set<YLanguage> possibleLanguages) {
		try {
			Detector detector = DetectorFactory.create(0.5f);
			detector.append(text);
			return detectLanguage(possibleLanguages, detector);
		} catch (LangDetectException | IOException e) {
			log.debug("Couldn't determine content language", e);
			return YLanguage.Undetermined;
		}
	}

	private static YLanguage detectLanguage(Set<YLanguage> possibleLanguages, Detector detector) throws LangDetectException {
		for (Language language : detector.getProbabilities()) {
			YLanguage lang = YLanguage.byCode(language.lang);
			if (language.prob > MINIMUM_PROBABILITY) {
				return lang;
			}
			if(language.prob> MINIMUM_PROBABILITY_FROM_SET && possibleLanguages.contains(lang)) {
				return lang;
			}
		}
		return YLanguage.Undetermined;
	}
}
